@@ -18,7 +18,16 @@ vcsExtract <- function(object){
 #### =========== ####
 "summary.mmer" <- function(object, ...) {
   
-  if(!object$reshape.output){stop("summary function only works for reshaped output.", call. = FALSE)}
+  replace.values <- function(Values,Search,Replace){
+    dd0 <- data.frame(Values)
+    vv <- which(Values%in%Search)
+    dd <- data.frame(Search,Replace)
+    rownames(dd) <- Search
+    dd0[vv,"Values"] <- as.character(dd[Values[vv],"Replace"])
+    return(dd0[,1])
+  }
+  
+  if(!object$reshapeOutput){stop("summary function only works for reshaped output.", call. = FALSE)}
   #dim(object$u.hat)
   digits = max(3, getOption("digits") - 3)
   #forget <- length(object)
@@ -143,55 +152,85 @@ vcsExtract <- function(object){
   ################################################
 }
 
-#### =========== ######
-## RESIDUALS FUNCTION #
-#### =========== ######
-"residuals.mmer" <- function(object, ...) {
+"summary.mmec" <- function(object, ...) {
+  
+  replace.values <- function(Values,Search,Replace){
+    dd0 <- data.frame(Values)
+    vv <- which(Values%in%Search)
+    dd <- data.frame(Search,Replace)
+    rownames(dd) <- Search
+    dd0[vv,"Values"] <- as.character(dd[Values[vv],"Replace"])
+    return(dd0[,1])
+  }
+  
   digits = max(3, getOption("digits") - 3)
   
-  pp <- fitted.mmer(object)
-  responses <- object$terms$response[[1]]
-  dat <- pp$dataWithFitted
-  for(ir in 1:length(responses)){
-    dat[,paste0(responses[ir],".residuals")] = dat[,responses[ir]] - dat[,paste0(responses[ir],".fitted")]
-  }
-
-  return(dat)
-}
-
-"print.residuals.mmer"<- function(x, digits = max(3, getOption("digits") - 3), ...) {
-  print((x))
-}
-#### =========== ######
-## RANEF FUNCTION #
-#### =========== ######
-
-"randef" <- function(object) {
-  # if(class(object)=="mmer"){
-  #   digits = max(3, getOption("digits") - 3)
-  #   cat("Returning object of class 'list' where each element correspond to one random effect.")
-  #   output <- object$u.hat
-  # }else{
-  #   stop("Class not recognized.\n",call. = FALSE)
-  # }
-  output<- object$U
+  lll <- object$llik
+  lll2 <- lll[length(lll)]
+  
+  LLAIC <- data.frame(as.numeric(lll2), as.numeric(object$AIC),
+                      as.numeric(object$BIC), "AI", object$convergence)
+  colnames(LLAIC) = c("logLik","AIC","BIC","Method","Converge")
+  rownames(LLAIC) <- "Value"
+  method="AI"
+  coef <- data.frame(Estimate=object$b)
+  
+  ## se and t values for fixed effects
+  nX <- length(object$b)
+  VarBeta <- object$Ci[1:nX,1:nX]
+  s2.beta <- diag(as.matrix(VarBeta))
+  coef$Std.Error <- sqrt(abs(s2.beta))
+  coef$t.value <- coef$Estimate/coef$Std.Error
+  
+  mys2 <- object$monitor[,which(object$llik[1,] == max(object$llik[1,]))] 
+  varcomp <- as.data.frame(cbind(mys2,sqrt(diag(solve(object$avInf/2)))))
+  varcomp[,3] <- varcomp[,1]/varcomp[,2]
+  colnames(varcomp) <- c("VarComp","VarCompSE","Zratio")
+  
+  varcomp$Constraint <- replace.values(object$constraints, 1:3, c("Positive","Unconstr","Fixed"))
+  
+  output <- list(varcomp=varcomp, betas=coef, method=method,logo=LLAIC)
+  attr(output, "class")<-c("summary.mmec", "list")
   return(output)
 }
 
-#### =========== ######
-## FIXEF FUNCTION #
-#### =========== ######
-
-
-#"fixef.mmer" <- function(object, ...) {
-#  digits = max(3, getOption("digits") - 3)
-#  output <- object$beta.hat
-#  return(output)
-#}
-
-#"print.fixef.mmer"<- function(x, digits = max(3, getOption("digits") - 3), ...) {
-#  print((x))
-#}
+"print.summary.mmec"<-function (x, digits = max(3, getOption("digits") - 3),  ...){
+  
+  nmaxchar0 <- max(as.vector(unlist(apply(data.frame(rownames(x$varcomp)),1,nchar))),na.rm = TRUE)
+  
+  if(nmaxchar0 < 26){
+    nmaxchar0 <- 26
+  } # + 26 spaces we have nmaxchar0+26  spaces to put the title
+  
+  nmaxchar <- nmaxchar0+34 ## add spaces from the 3 columns
+  nmaxchar2 <- nmaxchar0+18
+  nmaxchar3 <- nmaxchar0+34-46 #round(nmaxchar0/2)
+  rlh <- paste(rep("*",round(nmaxchar2/2)),collapse = "")
+  rlt <- paste(rep(" ",ceiling(nmaxchar3/2)),collapse = "")
+  digits = max(3, getOption("digits") - 3)
+  ################################################
+  cat(paste(rep("=",nmaxchar), collapse = ""))
+  cat(paste("\n",rlt,"Multivariate Linear Mixed Model fit by REML",rlt,"\n", collapse = ""))
+  cat(paste(rlh," sommer 4.1 ",rlh, "\n", collapse = ""))
+  cat(paste(rep("=",nmaxchar), collapse = ""))
+  cat("\n")
+  cat("")
+  print(x$logo)#, digits = digits)
+  cat(paste(rep("=",nmaxchar), collapse = ""))
+  cat("\nVariance-Covariance components:\n")
+  print(x$varcomp, digits = digits)
+  cat(paste(rep("=",nmaxchar), collapse = ""))
+  cat("\nFixed effects:\n")
+  if(nrow(x$betas) > 8){
+    print(x$betas[1:8,], digits = digits)
+    cat("   ... please access the object to see more\n")
+  }else{
+    print(x$betas, digits = digits)
+  }
+  cat(paste(rep("=",nmaxchar), collapse = ""))
+  cat("\nUse the '$' sign to access results and parameters")#\nArguments set to FALSE for multiresponse models:\n'draw', and 'gwas.plots'\n")
+  ################################################
+}
 
 #### =========== ####
 ## FITTED FUNCTION ##
@@ -203,34 +242,34 @@ vcsExtract <- function(object){
     originalModelForMatrices <- mmer(fixed=object$call$fixed,
                                      # random=object$call$random,
                                      rcov=object$call$rcov,
-                                     data=object$dataOriginal, return.param = TRUE,#reshape.output =FALSE,
+                                     data=object$dataOriginal, returnParam = TRUE,#reshapeOutput =FALSE,
                                      init = object$sigma_scaled, constraints = object$constraints,
-                                     na.method.Y = object$call$na.method.Y,
-                                     na.method.X = object$call$na.method.X,...)
+                                     naMethodY = object$call$naMethodY,
+                                     naMethodX = object$call$naMethodX,...)
     
     originalModelForParameters <- mmer(fixed=object$call$fixed,
                                        # random=object$call$random,
-                                       rcov=object$call$rcov, iters=1, verbose=FALSE,
-                                       data=object$dataOriginal, reshape.output =FALSE,
+                                       rcov=object$call$rcov, nIters=1, verbose=FALSE,
+                                       data=object$dataOriginal, reshapeOutput =FALSE,
                                        init = object$sigma_scaled, constraints = object$constraints,
-                                       na.method.Y = object$call$na.method.Y,
-                                       na.method.X = object$call$na.method.X,...)
+                                       naMethodY = object$call$naMethodY,
+                                       naMethodX = object$call$naMethodX,...)
   }else{
     originalModelForMatrices <- mmer(fixed=object$call$fixed,
                                      random=object$call$random,
                                      rcov=object$call$rcov,
-                                     data=object$dataOriginal, return.param = TRUE,#reshape.output =FALSE,
+                                     data=object$dataOriginal, returnParam = TRUE,#reshapeOutput =FALSE,
                                      init = object$sigma_scaled, constraints = object$constraints,
-                                     na.method.Y = object$call$na.method.Y,
-                                     na.method.X = object$call$na.method.X,...)
+                                     naMethodY = object$call$naMethodY,
+                                     naMethodX = object$call$naMethodX,...)
     
     originalModelForParameters <- mmer(fixed=object$call$fixed,
                                        random=object$call$random,
-                                       rcov=object$call$rcov, iters=1, verbose=FALSE,
-                                       data=object$dataOriginal, reshape.output =FALSE,
+                                       rcov=object$call$rcov, nIters=1, verbose=FALSE,
+                                       data=object$dataOriginal, reshapeOutput =FALSE,
                                        init = object$sigma_scaled, constraints = object$constraints,
-                                       na.method.Y = object$call$na.method.Y,
-                                       na.method.X = object$call$na.method.X,...)
+                                       naMethodY = object$call$naMethodY,
+                                       naMethodX = object$call$naMethodX,...)
   }
   ys <- object$terms$response[[1]]
   nt <- length(ys) # number of traits
@@ -285,18 +324,60 @@ vcsExtract <- function(object){
   head(x$dataWithFitted,...)
 }
 
-"head.fitted.mmer"<- function(x, digits = max(3, getOption("digits") - 3), ...) {
+"fitted.mmec" <- function(object,...){
+  
+  ff <- object$W %*% object$bu
+  
+  return(ff)
+}
+
+"print.fitted.mmec"<- function(x, digits = max(3, getOption("digits") - 3), ...) {
   cat(blue(paste("\n  The fitted values are obtained by adding Xb + Zu.1 + ... + Zu.n
                  containing: \n")
   ))
-  print(x$fittedSummary)
   cat(blue(paste("\n  head of fitted values: \n")
   ))
-  head(x$dataWithFitted,...)
+  head(x,...)
 }
 
+#### =========== ######
+## RESIDUALS FUNCTION #
+#### =========== ######
+"residuals.mmer" <- function(object, ...) {
+  digits = max(3, getOption("digits") - 3)
+  
+  pp <- fitted.mmer(object)
+  responses <- object$terms$response[[1]]
+  dat <- pp$dataWithFitted
+  for(ir in 1:length(responses)){
+    dat[,paste0(responses[ir],".residuals")] = dat[,responses[ir]] - dat[,paste0(responses[ir],".fitted")]
+  }
+  
+  return(dat)
+}
 
+"print.residuals.mmer"<- function(x, digits = max(3, getOption("digits") - 3), ...) {
+  print((x))
+}
 
+"residuals.mmec" <- function(object, ...) {
+  digits = max(3, getOption("digits") - 3)
+  ff <- fitted.mmec(object)
+  e <- object$y - ff
+  return(e)
+}
+
+"print.residuals.mmec"<- function(x, digits = max(3, getOption("digits") - 3), ...) {
+  print((x))
+}
+#### =========== ######
+## RANEF FUNCTION #
+#### =========== ######
+
+"randef" <- function(object) {
+  output<- object$U
+  return(output)
+}
 
 #### =========== ####
 ## COEF FUNCTION ####
@@ -310,6 +391,14 @@ vcsExtract <- function(object){
   print((x))
 } 
 
+"coef.mmec" <- function(object, ...){
+  object$b
+}
+
+"print.coef.mmec"<- function(x, digits = max(3, getOption("digits") - 3), ...) {
+  print((x))
+} 
+
 #### =========== ####
 ## ANOVA FUNCTION ###
 #### =========== ####
@@ -369,33 +458,33 @@ anova.mmer <- function(object, object2=NULL, type=1, ...) {
       if(is.null(object$call$random)){
         prov0 <- mmer(fixed=fixedi,
                       # random = object$call$random,
-                      rcov=object$call$rcov, iters=1,
-                      data=object$data, return.param = FALSE,reshape.output=FALSE,
+                      rcov=object$call$rcov, nIters=1,
+                      data=object$data, returnParam = FALSE,reshapeOutput=FALSE,
                       verbose = FALSE,
                       init = object$sigma_scaled, constraints = object$constraints,
-                      na.method.Y = object$call$na.method.Y, 
-                      na.method.X = object$call$na.method.X)
+                      naMethodY = object$call$naMethodY, 
+                      naMethodX = object$call$naMethodX)
         prov <- mmer(fixed=fixedi,
                      # random = object$call$random,
                      rcov=object$call$rcov,
-                     data=object$data, return.param = TRUE,
-                     na.method.Y = object$call$na.method.Y, 
-                     na.method.X = object$call$na.method.X)
+                     data=object$data, returnParam = TRUE,
+                     naMethodY = object$call$naMethodY, 
+                     naMethodX = object$call$naMethodX)
       }else{
         prov0 <- mmer(fixed=fixedi,
                       random = object$call$random,
-                      rcov=object$call$rcov, iters=1,
-                      data=object$data, return.param = FALSE,reshape.output=FALSE,
+                      rcov=object$call$rcov, nIters=1,
+                      data=object$data, returnParam = FALSE,reshapeOutput=FALSE,
                       verbose = FALSE,
                       init = object$sigma_scaled, constraints = object$constraints,
-                      na.method.Y = object$call$na.method.Y, 
-                      na.method.X = object$call$na.method.X)
+                      naMethodY = object$call$naMethodY, 
+                      naMethodX = object$call$naMethodX)
         prov <- mmer(fixed=fixedi,
                      random = object$call$random,
                      rcov=object$call$rcov,
-                     data=object$data, return.param = TRUE,
-                     na.method.Y = object$call$na.method.Y, 
-                     na.method.X = object$call$na.method.X)
+                     data=object$data, returnParam = TRUE,
+                     naMethodY = object$call$naMethodY, 
+                     naMethodX = object$call$naMethodX)
       }
       Y <- prov[[1]]
       n= nrow(Y)
@@ -513,6 +602,56 @@ anova.mmer <- function(object, object2=NULL, type=1, ...) {
   }
   return(result)
 }
+
+anova.mmec <- function(object, object2=NULL, ...) {
+  signifo <- function(x){
+    if(x >= 0 & x < 0.001){y="***"}
+    if(x >= 0.001 & x < 0.01){y="**"}
+    if(x >= 0.01 & x < 0.05){y="*"}
+    if(x >= 0.05 & x < 0.1){y="."}
+    if(x > 0.1){y=""}
+    return(y)
+  }
+  ########################################
+  digits = max(3, getOption("digits") - 3)
+  if(is.null(object2)){
+    stop("The 'anova' function for the sommer package only works to compare mixed models by likelihood ratio tests (LRT), was not intended to provide regular sum of squares output.")
+    # result <- sequential.fit(object,type=type)
+  }else{
+    dis=c(
+          nrow(object$monitor)+nrow(object$b),
+          nrow(object2$monitor)+nrow(object2$b)
+          ) # dimensions
+    mods=c("mod1","mod2")
+    lls=c( object$llik[ncol(object$llik)],  object2$llik[ncol(object2$llik)] ) # likelihoods
+    aics=c(object$AIC, object2$AIC) # AIC's
+    bics=c(object$BIC, object2$BIC) # AIC's
+    vv=which(dis == max(dis))[1] # which has more variance components BIGGER
+    vv2=c(1:2)[which(c(1:2)!= vv)] # SMALLER
+    LR = (lls[vv] - lls[vv2])
+    r.stat= abs(-2*((LR))) # -2(LL1 - LL2)
+    df=dis[vv]-dis[vv2]
+    chichi=pchisq((r.stat), df, lower.tail=FALSE)
+    if(chichi > 1e-5){
+      chichi <- round(chichi,5)
+    }
+    chichi2=paste(as.character(chichi),signifo(chichi), sep=" ")
+    ### construct the table
+    cat("Likelihood ratio test for mixed models\n")
+    cat("==============================================================\n")
+    result=data.frame(Df=c(dis[vv],dis[vv2]), AIC=c(aics[vv],aics[vv2]), 
+                      BIC=c(bics[vv],bics[vv2]), loLik=c(lls[vv],lls[vv2]), 
+                      Chisq=c("",as.character(round(r.stat,5))), 
+                      ChiDf=c("",as.character(df)), PrChisq=c("",chichi2 ))
+    rownames(result) <- c(mods[vv],mods[vv2])
+    print(result)
+    cat("==============================================================\n")
+    cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n")
+    
+    #}
+  }
+  return(result)
+}
 #### =========== ####
 ## PLOTING FUNCTION #
 #### =========== ####
@@ -526,9 +665,9 @@ plot.mmer <- function(x, stnd=TRUE, ...) {
   prov <- mmer(fixed=x$call$fixed,
                #random=x$call$random,
                rcov=x$call$rcov,
-               data=x$data, return.param = TRUE,#reshape.results=TRUE,
-               na.method.Y = x$call$na.method.Y, 
-               na.method.X = x$call$na.method.X)
+               data=x$data, returnParam = TRUE,#reshape.results=TRUE,
+               naMethodY = x$call$naMethodY, 
+               naMethodX = x$call$naMethodX)
   Xlist <- list()
   for(o in 1:length(prov[[3]])){
     Xlist[[o]] <- kronecker(prov[[2]][[o]],prov[[3]][[o]])
@@ -553,6 +692,28 @@ plot.mmer <- function(x, stnd=TRUE, ...) {
   layout(matrix(1,1,1))
 }
 
+plot.mmec <- function(x, stnd=TRUE, ...) {
+  digits = max(3, getOption("digits") - 3)
+  transp <- function (col, alpha = 0.5){
+    res <- apply(col2rgb(col), 2, function(c) rgb(c[1]/255, c[2]/255,c[3]/255, alpha))
+    return(res)
+  }
+  layout(matrix(1:4,2,2))
+  # ff <- fitted(x)
+  rr <- residuals.mmec(x)
+  # for(i in 1:traits){
+    
+    plot(rr,scale(rr),pch=20, col=transp("cadetblue"), ylab="Std Residuals", xlab="Fitted values", main="Residual vs Fitted", bty="n", ...); grid()
+    plot(rr,sqrt(abs(scale(rr))),pch=20, col=transp("thistle4"), ylab="Sqrt Abs Std Residuals", xlab="Fitted values", main="Scale-Location", bty="n", ...); grid()
+    
+    qqnorm(scale(rr), pch=20, col=transp("tomato1"), ylab="Std Residuals", bty="n",...); grid()
+    # hat <- Xm%*%solve(t(Xm)%*%x$Vi%*%Xm)%*%t(Xm)%*%x$Vi # leverage including variance from random effects H= X(X'V-X)X'V-
+    hat = x$W %*% x$Ci %*% t(x$W)
+    plot(diag(hat), scale(rr), pch=20, col=transp("blue"), ylab="Std Residuals", xlab="Leverage", main="Residual vs Leverage", bty="n", ...); grid()
+  # }
+  #####################
+  layout(matrix(1,1,1))
+}
 ##################################################################################################
 #Startup function
 #this function is executed once the library is loaded
@@ -563,148 +724,24 @@ plot.mmer <- function(x, stnd=TRUE, ...) {
     stop("This package requires R 2.1 or later")
   assign(".sommer.home", file.path(library, pkg),
          pos=match("package:sommer", search()))
-  sommer.version = "4.1.6 (2022-04-01)" # usually 2 months before it expires
+  sommer.version = "4.1.7 (2022-07-01)" # usually 2 months before it expires
   
   assign(".sommer.version", sommer.version, pos=match("package:sommer", search()))
   if(interactive())
   {
-    packageStartupMessage(yellow(paste("[]==================================================================[]")),appendLF=TRUE)
-    packageStartupMessage(yellow(paste("[]   Solving Mixed Model Equations in R (sommer) ", sommer.version, " []",sep="")),appendLF=TRUE)
-    packageStartupMessage(yellow(paste("[]   ------------- Multivariate Linear Mixed Models --------------  []")),appendLF=TRUE)
-    packageStartupMessage(yellow("[]   Author: Giovanny Covarrubias-Pazaran                           []"),appendLF=TRUE)
-    packageStartupMessage(yellow("[]   Published: PLoS ONE 2016, 11(6):1-15                           []"),appendLF=TRUE)
-    packageStartupMessage(yellow("[]   Dedicated to the University of Chapingo and UW-Madison         []"),appendLF=TRUE)
-    packageStartupMessage(yellow("[]   Type 'vignette('v1.sommer.quick.start')' for a short tutorial  []"),appendLF=TRUE)
-    packageStartupMessage(yellow("[]   Type 'citation('sommer')' to know how to cite sommer           []"),appendLF=TRUE)
-    packageStartupMessage(yellow(paste("[]==================================================================[]")),appendLF=TRUE)
-    packageStartupMessage(yellow("sommer is updated on CRAN every 4-months due to CRAN policies"),appendLF=TRUE)
-    packageStartupMessage(yellow("Newest source is available at https://github.com/covaruber/sommer"),appendLF=TRUE)
-    packageStartupMessage(yellow("To install type: library(devtools); install_github('covaruber/sommer')"),appendLF=TRUE)
+    packageStartupMessage(blue(paste("[]==================================================================[]")),appendLF=TRUE)
+    packageStartupMessage(blue(paste("[]   Solving Mixed Model Equations in R (sommer) ", sommer.version, " []",sep="")),appendLF=TRUE)
+    packageStartupMessage(blue(paste("[]   ------------- Multivariate Linear Mixed Models --------------  []")),appendLF=TRUE)
+    packageStartupMessage(blue("[]   Author: Giovanny Covarrubias-Pazaran                           []"),appendLF=TRUE)
+    packageStartupMessage(blue("[]   Published: PLoS ONE 2016, 11(6):1-15                           []"),appendLF=TRUE)
+    packageStartupMessage(blue("[]   Dedicated to the University of Chapingo and UW-Madison         []"),appendLF=TRUE)
+    packageStartupMessage(blue("[]   Type 'vignette('v1.sommer.quick.start')' for a short tutorial  []"),appendLF=TRUE)
+    packageStartupMessage(blue("[]   Type 'citation('sommer')' to know how to cite sommer           []"),appendLF=TRUE)
+    packageStartupMessage(blue(paste("[]==================================================================[]")),appendLF=TRUE)
+    packageStartupMessage(blue("sommer is updated on CRAN every 4-months due to CRAN policies"),appendLF=TRUE)
+    packageStartupMessage(blue("Newest source is available at https://github.com/covaruber/sommer"),appendLF=TRUE)
+    packageStartupMessage(blue("To install type: library(devtools); install_github('covaruber/sommer')"),appendLF=TRUE)
     
-    #if(yyy > current){ # yyy < current in CRAN
-    #  packageStartupMessage(paste("Version",current,"is now available."),appendLF=TRUE) # version current
-    #  packageStartupMessage(paste("Please update 'sommer' installing the new version."),appendLF=TRUE) # version current
-    #}
   }
   invisible()
 }
-
-#.onLoad <- function(library, pkg) {
-#  data(x)
-#  library(audio)
-#  packageStartupMessage(play(x))
-#}
-
-# plot.variogram.mmer <- function(x, stnd=TRUE, ...) {
-#   
-#   #for(u in 1:length(x)){
-#   
-#   x0 <- x#[[u]]
-#   min.length = 30
-#   if(stnd){x0$data$value <- scale(x0$data$value)}
-#   values <- matrix(replace(x0$data$value, x0$data$length < min.length,
-#                            NA), ncol = length(x0$col.displacement), nrow = length(x0$row.displacement),
-#                    byrow = TRUE)
-#   
-#   print(wireframe(values,drape=TRUE, #main=names(x)[u],
-#                   aspect = c(61/87, 0.4), #colorkey=TRUE,
-#                   light.source = c(10,0,10), #shade=TRUE,
-#                   #screen = list(x0 = -60, y = 50, z=20),
-#                   col.regions = topo.colors(100))
-#   )
-#   # light.source = c(0,0,10),
-#   # #region = TRUE,
-#   # col.regions = terrain.colors(100),
-#   # screen = list(x0 = -60, y = 50, z=20)))
-#   
-#   # persp(x0, y, z, theta = 135, phi = 30, col = colorRampPalette(c("blue", "pink"))(9500), scale = FALSE,
-#   #       ltheta = -120, shade = 0.75, border = NA, box = FALSE)
-#   
-#   #}
-#   # plot3Drgl::persp3Drgl(x0$row.displacement, x0$col.displacement,
-#   #                       values, xlab = "Row displacement", ylab = "Col displacement",
-#   #                       zlab = "", ticktype = "detailed", col = jet.colors(100))
-# }
-# 
-# variogram.mmer <- function (x, xcoor="ROW", ycoor="RANGE", zcoor=NULL, by=NULL, ...){
-#   #x is a dataset with columns Residuals, xcoor, ycoor
-#   # xcoor is the name of column for the xcoordinate
-#   # ycoor is the name of column for the ycoordinate
-#   #library(data.table)
-#   
-#   x0 <- x$data
-#   
-#   if(is.null(by)){
-#     x0$FIELDINST <- "F1"
-#     x0$FIELDINST <- as.factor(x0$FIELDINST)
-#     by="FIELDINST"
-#   }else{
-#     v <- which(colnames(x0) == by)
-#     if(length(v)==0){stop("by argument not found in the column names of x0", call. = FALSE)}
-#     x0[,by] <- as.factor(x0[,by])
-#   }
-#   
-#   if(is.null(zcoor)){
-#     zcoor <- "Residuals"
-#     x0[,zcoor] <- x$res.ordim
-#   }else{
-#     www <- which(names(x$Zus) %in% zcoor)
-#     if(length(www)==1){
-#       x0[,zcoor] <- x$Zus[[zcoor]]
-#     }else if(length(www)>1){
-#       zcoor <- paste(zcoor, collapse = "_")
-#       gggg <- Reduce("+",x$Zus[www])
-#       x0[,zcoor] <- gggg
-#     }else{
-#       stop("zcoor not found in your model", call. = FALSE)
-#     }
-#   }
-#   
-#   are <- which(colnames(x0) %in% c(xcoor,ycoor,zcoor))
-#   if(length(are) < 3){
-#     stop("One or more of xcoor, ycoor, zcoor don't match your data frame.", call. = FALSE)
-#   }
-#   
-#   ## now add the zcoor
-#   
-#   x1<- split(x0, f=x0[,by])
-#   
-#   multires <- lapply(x1, function(x){
-#     x.coord <- x[, xcoor]
-#     y.coord <- x[, ycoor]
-#     residuals <- x[, zcoor]
-#     columns <- seq(min(x.coord), max(x.coord), by = min(diff(sort(unique(x.coord)))))
-#     rows <- seq(min(y.coord), max(y.coord), by = min(diff(sort(unique(y.coord)))))
-#     xy.coord <- data.table(expand.grid(columns = columns, rows = rows))
-#     setkeyv(xy.coord, c("rows", "columns"))
-#     df <- data.table(columns = x.coord, rows = y.coord, residuals = residuals)
-#     setkeyv(df, c("rows", "columns"))
-#     df <- df[xy.coord]
-#     df <- df[order(df$columns, df$rows), ]
-#     resdiff <- c(outer(df$residuals, df$residuals, function(x, 
-#                                                             y) 0.5 * (x - y)^2))
-#     coldiff <- c(outer(df$columns, df$columns, function(x, y) abs(x - 
-#                                                                     y)))
-#     coldiff.u <- unique(coldiff)
-#     rowdiff <- c(outer(df$rows, df$rows, function(x, y) abs(x - 
-#                                                               y)))
-#     rowdiff.u <- unique(rowdiff)
-#     subsets <- split(resdiff, f = list(coldiff, rowdiff))
-#     value <- sapply(subsets, mean, na.rm = TRUE)
-#     length <- sapply(subsets, function(x) sum(!is.na(x)))
-#     length[-1] <- length[-1]/2
-#     res <- list(data = data.frame(value = value, length = length), 
-#                 col.displacement = coldiff.u, row.displacement = rowdiff.u)
-#     class(res) <- "variogram.mmer"
-#     return(res)
-#   })
-#   #print(str(multires))
-#   # multires <- as.data.frame(do.call(rbind,multires))
-#   # class(multires) <- "variogram.sommer"
-#   # 
-#   # funny <- as.formula(paste(zcoor,"~",xcoor,"*",ycoor,"|",by, sep="" ))
-#   # multires2 <- list(data=multires,funny=funny)
-#   class(multires) <- "variogram.mmer"
-#   
-#   return(multires)
-# }
